/*
 * begin.cxx
 *
 * Copyright 2011-2022 Micke Nordin <hej@mic.ke>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 */

#include <boost/iostreams/copy.hpp>
#include <boost/iostreams/filter/bzip2.hpp>
#include <boost/iostreams/filter/gzip.hpp>
#include <boost/iostreams/filtering_streambuf.hpp>
#include <boost/xpressive/xpressive.hpp>
#include <fstream>
#include <iostream>
#include <string>

using namespace std;

int begin(string needle, istream &in) {
  string input; // This is where we keep our input, line by line
  boost::xpressive::sregex regex = boost::xpressive::sregex::compile(
      needle,
      boost::xpressive::regex_constants::icase); // this is what we search for
  if (!in.good()) {                              // Check if steam is ok
    cerr << "Could not open file \n";
    return 1;    // if not return 1
  } else {       // go ahead
    while (in) { // loop through input line by line
      getline(in, input);
      if (boost::xpressive::regex_search(input, regex)) { // if we find a match
        cout << input << endl; // start printing to std out
        while (in) {           // and print what remains as well
          getline(in, input);
          cout << input << endl;
        } // end of inner while
      }   // end of if
    }     // end of outer while
  }       // end of else
  return 0;
}
int begin(string needle, string haystack) {
  istringstream iss(haystack);
  istream &in(iss);
  return begin(needle, in);
}

string gzip_decomp(string filename) {
  ifstream file(filename, ios_base::in | ios_base::binary);
  stringstream compressed;
  compressed << file.rdbuf();
  stringstream decompressed;

  boost::iostreams::filtering_streambuf<boost::iostreams::input> out;
  out.push(boost::iostreams::gzip_decompressor());
  out.push(compressed);
  boost::iostreams::copy(out, decompressed);

  return decompressed.str();
}

string bzip_decomp(string filename) {
  ifstream file(filename, ios_base::in | ios_base::binary);
  stringstream compressed;
  compressed << file.rdbuf();
  stringstream decompressed;

  boost::iostreams::filtering_streambuf<boost::iostreams::input> out;
  out.push(boost::iostreams::bzip2_decompressor());
  out.push(compressed);
  boost::iostreams::copy(out, decompressed);

  return decompressed.str();
}
